/*------------------------------------------------*/
/* 21zakladyII/ackermann.c                        */

#include <sys/time.h>
#include <sys/resource.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>

typedef unsigned long long int llu;

static llu rekurze = 0;
static llu rmax = 0;
static llu ackcalls  = 0;

void setStack() {
    	const rlim_t kStackSize = 0.25 * 1024 * 1024;   // stack size
    	struct rlimit rl;
    	int result;

    	result = getrlimit(RLIMIT_STACK, &rl);
    	if (result != 0) {
		perror(NULL);
		abort();
	}
    	printf("current stack limit = %.2f MiB\n",
		(double) rl.rlim_cur/(1024*1024));
	printf("set stack limit to %.2f MiB\n",
                (double) kStackSize/(1024*1024));
	rl.rlim_cur = kStackSize;
        result = setrlimit(RLIMIT_STACK, &rl);
        if (result != 0)
        {
		fprintf(stderr, "setrlimit returned result = %d\n", result);
		abort();
        }
}

int ack(llu m,llu n)
{
	llu ans;
	rekurze++;
	ackcalls++;
	rmax = rmax > rekurze ? rmax : rekurze;
	printf("\rRekurze: %7llu; rmax = %7llu, calls = %llu", rekurze, rmax, ackcalls);
	if (m == 0) ans = n+1;
	else if (n == 0) ans = ack(m-1,1);
	else ans = ack(m-1, ack(m,n-1));
	rekurze--;
	return ans;
}


int main (int argc,char ** argv)
{ 
	time_t start, end;
	int i,j;
	setStack();

	for (i=0; i<=4; i++) {
		for (j=0;j<=4; j++) {
			ackcalls = rmax = 0;
			start = time(NULL);
			llu ans = ack(i, j);
			end = time(NULL);
			double diff = difftime(end, start);
			printf (
				"\rackerman (%d,%d) is: %6llu. "
				"duration = %2.0lfm %2.0lfs., "
				"rmax = %5llu, calls = %10llu\n",
				i,j, ans, 
				diff/60, diff - ((llu) diff/60)*60,
				rmax, ackcalls
			);
		}
	}
	return 0;
}

/*------------------------------------------------*/
