/*------------------------------------------------*/
/* 28dbm/gdbm.c                                   */

#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <gdbm.h>

#define ULOZISTE "uloziste"
#define MAX_LEN 512
#define POLOZEK 4

struct text {
    int id;
    char text[MAX_LEN];
};

void printResult(datum * key, datum * result);

int main(void)
{
    GDBM_FILE dbm;
    datum key, nextkey;
    datum content;
    datum result;
    int i;
    struct text polozka;
    char *klice[POLOZEK] = { "1", "2", "3", "3" };
    char *texty[POLOZEK] = { "jedna", "dva", "tři", "čtyři" };

    /* otevreni database */
    dbm = gdbm_open(ULOZISTE, 0, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR, NULL);
    if (!dbm) {
        fprintf(stderr, "%s\n", gdbm_strerror(gdbm_errno));
        return 1;
    }

    /* ulozeni polozek */
    for (i = 0; i < POLOZEK; i++) {
        key.dptr = (void *)klice[i];
        key.dsize = strlen(klice[i]) + 1;

        polozka.id = i + 1;
        strncpy(polozka.text, texty[i], MAX_LEN - 1);
        polozka.text[MAX_LEN - 1] = '\0';

        content.dptr = (void *)&polozka;
        content.dsize = sizeof(polozka.id) + strlen(polozka.text) + 1;

        if (gdbm_store(dbm, key, content, GDBM_INSERT)) {
            fprintf(stderr, "dbm_store: %s\n",
                gdbm_strerror(gdbm_errno));
        }
    }

    /* nacteni a vypsani polozky "3" */
    key.dptr = (void *)"3";
    key.dsize = 2;        /* '3' a '\0' */
    result = gdbm_fetch(dbm, key);
    printResult(&key, &result);
    /* nacteni a vypsani polozky "4" */
    key.dptr = (void *)"4";
    key.dsize = 2;        /* '3' a '\0' */
    result = gdbm_fetch(dbm, key);
    printResult(&key, &result);

    /* nacteni a vypsani vsech polozek */
    printf("***\n");
    key = gdbm_firstkey(dbm);
    while (key.dptr) {
        result = gdbm_fetch(dbm, key);
        printResult(&key, &result);

        nextkey = gdbm_nextkey(dbm, key);
        free(key.dptr);
        key = nextkey;
    }

    gdbm_close(dbm);

    return 0;
}

/**
* Zkontroluje, zda neni result prazdny
* a pak jej vytiskne.
* Nakonec uvolni pamet alokovanou pro result.
*/
void printResult(datum * key, datum * result)
{
    struct text r;

    if (!result->dptr) {
        fprintf(stderr, "polozka[%s]: %s\n", (char *)key->dptr,
            gdbm_strerror(gdbm_errno));
        return;
    }
    memcpy(&r, result->dptr, result->dsize);
    printf("polozka[%s]: [%i, %s]\n", (char *)key->dptr, r.id, r.text);
    free(result->dptr);
}

/*------------------------------------------------*/
