/*------------------------------------------------*/
/* 29postgresql/pg-utils.c                        */
#include <string.h>
#include "pg-utils.h"

void usage()
{
    printf("connect dbname username [password]\n");
}

PGconn *pgconnect(int argc, char **argv)
{
    PGconn *conn;
    const char *const keywords[] =
        { "dbname", "user", "password", "client_encoding", NULL };
    const char *values[5];

    if (argc < 3) {
        usage();
        return NULL;
    }

    values[0] = argv[1];
    values[1] = argv[2];
    values[2] = (argc > 3) ? argv[3] : "";
    values[3] = "UTF8";
    values[4] = NULL;

    conn = PQconnectdbParams(keywords, values, 0);
    if (PQstatus(conn) != CONNECTION_OK) {
        fprintf(stderr, "Připojení k databázi selhalo: %s",
            PQerrorMessage(conn));
        PQfinish(conn);
        return NULL;
    }
    return conn;
}

ExecStatusType printStatus(PGresult * res)
{
    ExecStatusType status = PQresultStatus(res);
    switch (status) {
    case PGRES_EMPTY_QUERY:
        printf("Řetězec odeslaný serveru byl prázdný\n");
        break;
    case PGRES_COMMAND_OK:
        printf
            ("Příkaz byl úspěšně proveden. Nevrátil žádná data.\n");
        break;
    case PGRES_TUPLES_OK:
        printf
            ("Příkaz byl úspěšně proveden. Mohl vrátit data.\n");
        break;
        /* SINGLE TUPLe může nastat, jen když je zapnutý tzv. single-row mode */
    case PGRES_SINGLE_TUPLE:
        printf("Server vrátil jeden řádek\n");
        break;

    case PGRES_BAD_RESPONSE:
        printf("Odpověď od serveru byla nesrozumetelná\n");
    case PGRES_NONFATAL_ERROR:
        printf("Server vrátil NOTICE nebo WARNING\n");
    case PGRES_FATAL_ERROR:
        printf("Server vrátil falal error\n");
        printf("Error: %s\n", PQresultErrorMessage(res));
        break;
    case PGRES_COPY_OUT:
    case PGRES_COPY_IN:
    case PGRES_COPY_BOTH:
        printf("Probíhá přenos dat z nebo na server nebo obojí.\n");
        break;
    default:
        printf("Neznámý status\n");
    }
    return status;
}

char *getTypeName(PGconn * conn, PGresult * res, int column_number)
{
    PGresult *type_res;
    char sql[100];
    static char typname[65];
    Oid type = PQftype(res, column_number);
    if (type <= 0) {
        return NULL;
    }
    sprintf(sql, "SELECT typname FROM pg_catalog.pg_type  WHERE OID = %i",
        type);
    type_res = PQexec(conn, sql);
    strncpy(typname, PQgetvalue(type_res, 0, 0), sizeof(typname));
    typname[sizeof(typname) - 1] = '\0';
    return typname;
}

/*------------------------------------------------*/
