/*------------------------------------------------*/
/* 40socket/connection-pipe.c                     */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdbool.h>
#include <stdio.h>
#include <errno.h>
#include "connection.h"

#define PIPE_FILE_SERVER "/tmp/named_pipe_server"
#define PIPE_FILE_CLIENT "/tmp/named_pipe_client"

static bool vytvorRouru(char *name)
{
    if (mkfifo(name, 0666) == -1) {
        if (errno != EEXIST) {
            perror("mkfifo");
            return false;
        }
    }
    return true;
}

bool startServer(void)
{
    if (!vytvorRouru(PIPE_FILE_SERVER))
        return false;
    if (!vytvorRouru(PIPE_FILE_CLIENT))
        return false;
    return true;
}

void stopServer(void)
{
    unlink(PIPE_FILE_SERVER);
    unlink(PIPE_FILE_CLIENT);
}

descriptors waitForClientConnection(void)
{
    descriptors fd = { -1, -1 };

    fd.r = open(PIPE_FILE_CLIENT, O_RDONLY);
    if (fd.r < 1) {
        perror("open r");
        return fd;
    }
    fd.w = open(PIPE_FILE_SERVER, O_WRONLY);
    if (fd.w < 1) {
        perror("open w");
        return fd;
    }
    return fd;
}

void closeClientConnection(descriptors * fd)
{
    if (fd->r >= 0 && close(fd->r) < 0) {
        perror("close r");
    }
    if (fd->w >= 0 && close(fd->w) < 0) {
        perror("close w");
    }
    fd->r = fd->w = -1;
}

descriptors connectClientToServer(void)
{
    descriptors fd = { -1, -1 };

    fd.w = open(PIPE_FILE_CLIENT, O_WRONLY);
    if (fd.w < 1) {
        perror("open w");
        return fd;
    }
    fd.r = open(PIPE_FILE_SERVER, O_RDONLY);
    if (fd.r < 1) {
        perror("open r");
        return fd;
    }
    return fd;
}

void disconnectClientFromServer(descriptors * fd)
{
    if (close(fd->r) < 0) {
        perror("close r");
    }
    if (close(fd->w) < 0) {
        perror("close w");
    }
}
/*------------------------------------------------*/
