/*------------------------------------------------*/
/* 44http/connection-inet.c                       */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdbool.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include "connection.h"

#define SERVER_IPv4 "127.0.0.1"
#define SERVER_PORT 50505

#ifndef _SALLYX
#define CLIENT_SERVER_IPv4 "127.0.0.1"
#define CLIENT_SERVER_PORT 50505
#else
#define CLIENT_SERVER_IPv4 "46.28.105.66"
#define CLIENT_SERVER_PORT 80
#endif

static int server_fd = -1;

bool startServer(void)
{
    struct sockaddr_in server_address;
    int ret;
    server_fd = socket(AF_INET, SOCK_STREAM, 0);
    if (server_fd < 0) {
        perror("socket");
        return false;
    }

    int on = 1;
    ret =
        setsockopt(server_fd, SOL_SOCKET, SO_REUSEADDR, (const void *)&on,
               sizeof(on));

    struct linger linger;
    linger.l_onoff = 1;
    linger.l_linger = 30;
    ret =
        setsockopt(server_fd, SOL_SOCKET, SO_LINGER, (const char *)&linger,
               sizeof(linger));
    if (ret < 0)
        perror("SO_LINGER");

    memset(&server_address, 0, sizeof(server_address));
    server_address.sin_family = AF_INET;
    server_address.sin_addr.s_addr = htonl(INADDR_ANY);
    server_address.sin_port = htons(SERVER_PORT);
    ret = bind(server_fd, (struct sockaddr *)&server_address,
           sizeof(server_address));
    if (ret < 0) {
        perror("bind");
        return false;
    }
    listen(server_fd, 5);
    return true;
}

void stopServer(void)
{
    close(server_fd);
}

descriptors waitForClientConnection(void)
{
    descriptors fd = { -1, -1 };
    struct sockaddr_in client_address;
    int client_fd;
    socklen_t client_len;

    memset(&client_address, 0, sizeof(client_address));
    client_len = sizeof(client_address);
    client_fd =
        accept(server_fd, (struct sockaddr *)&client_address, &client_len);
    fd.r = fd.w = client_fd;
    return fd;
}

void closeClientConnection(descriptors * fd)
{
    if (fd->r >= 0) {
        if (close(fd->r) < 0) {
            perror("close r");
        }
    }
    fd->r = fd->w = -1;
}

descriptors connectClientToServer(void)
{
    struct sockaddr_in address;
    int sockfd;
    int ret;
    descriptors fd = { -1, -1 };

    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0) {
        perror("socket");
        return fd;
    }

    memset(&address, 0, sizeof(address));
    address.sin_family = AF_INET;
    address.sin_port = htons(CLIENT_SERVER_PORT);
    /*address.sin_addr.s_addr = inet_addr(SERVER_IPv4); */
    printf("Connecting to IP %s\n", CLIENT_SERVER_IPv4);
    if (inet_aton(CLIENT_SERVER_IPv4, &(address.sin_addr)) == 0) {
        perror("inet_aton");
        return fd;
    }
    ret = connect(sockfd, (struct sockaddr *)&address, sizeof(address));
    if (ret < 0)
        perror("connect");
    fd.r = fd.w = sockfd;

    return fd;
}

void disconnectClientFromServer(descriptors * fd)
{
    if (close(fd->r) < 0) {
        perror("close r");
    }
}
/*------------------------------------------------*/
