/*------------------------------------------------*/
/* 51ncurses/signal.c                             */
#include <stdio.h>
#include <ncurses.h>
#include <locale.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>

static void obnovTerminal()
{
    printw("Obnovuji nastavení terminálu\n");
    endwin();
}

static void initNcurses() {
    setlocale(LC_ALL, "");
    initscr();
    noecho();
    raw();
    atexit(obnovTerminal);

    if (!has_colors()) {
        fprintf(stderr, "Terminál nepodporuje barvy\n");
        exit(EXIT_FAILURE);
    }
    if (start_color() != OK) {
        fprintf(stderr, "Nepodařilo se inicializovat barvy\n");
        exit(EXIT_FAILURE);
    }
}

typedef struct {
    int lines;
    int cols;
    int line;
    int col;
} wPosition;

static wPosition getWindowPosition()
{
    wPosition p;
    p.lines = LINES / 2;
    p.cols = COLS / 2;
    p.line = LINES / 2 - LINES / 4;
    p.col = COLS / 2 - COLS / 4;
    return p;
}

static WINDOW *okno = NULL;
static int refreshed = 0;
static int pair_okno = 1;

static void prekresliOkno(wPosition *p)
{
    int i, j;
    wattrset(okno, COLOR_PAIR(pair_okno));
    for (i = 0; i < p->lines; i++)
        for (j = 0; j < p->cols; j++)
            mvwprintw(okno, i, j, "%c", ' ');
    box(okno, ACS_VLINE, ACS_HLINE);
    wmove(okno, 0, 0);
    wprintw(okno, "Nové okno");
    if(p->lines/2 - 1 >=  0)
    mvwprintw(okno, p->lines / 2 - 1, 1,
          "LINES = %i, COLS = %i, refreshed %i times.", LINES, COLS,
          refreshed);
    wrefresh(okno);
}

static void zpracuj_winch()
{
    refreshed++;
    endwin();
    refresh();
    erase();
    refresh();
    wPosition p = getWindowPosition();
    wresize(okno, p.lines, p.cols);
    mvwin(okno, p.line, p.col);
    prekresliOkno(&p);
}

static void nastavSignal()
{
    struct sigaction sa;
    sigset_t set;
    sigemptyset(&set);
    sa.sa_handler = zpracuj_winch;
    sa.sa_mask = set;
    sa.sa_flags = SA_RESTART;
    sa.sa_restorer = NULL;
    sigaction(SIGWINCH, &sa, NULL);
}

int main(void)
{
    initNcurses();
    curs_set(0);
    nastavSignal();

    /* nastaveni barvy pozadi a prekresleni cele obrazovky */
    assume_default_colors(COLOR_WHITE, COLOR_MAGENTA);
    erase();
    refresh();

    wPosition p = getWindowPosition();
    okno = newwin(p.lines, p.cols, p.line, p.col);

    /* nastaveni okna */
    init_pair(pair_okno, COLOR_RED, COLOR_WHITE);
    zpracuj_winch();

    (void)getch();
    return 0;
}

/*------------------------------------------------*/
